package com.qmth.teachcloud.common.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.teachcloud.common.bean.dto.CourseInfoDto;
import com.qmth.teachcloud.common.bean.params.BasicCourseParams;
import com.qmth.teachcloud.common.bean.result.BasicCourseResult;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicCourse;
import com.qmth.teachcloud.common.entity.SysOrg;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.OrgCenterTypeEnum;
import com.qmth.teachcloud.common.enums.OrgTypeEnum;
import com.qmth.teachcloud.common.mapper.BasicCourseMapper;
import com.qmth.teachcloud.common.service.BasicCourseService;
import com.qmth.teachcloud.common.service.BasicUserCourseService;
import com.qmth.teachcloud.common.service.SysOrgService;
import com.qmth.teachcloud.common.service.TeachcloudCommonService;
import com.qmth.teachcloud.common.util.ServletUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.Objects;
import java.util.Set;

/**
 * <p>
 * 课程表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class BasicCourseServiceImpl extends ServiceImpl<BasicCourseMapper, BasicCourse> implements BasicCourseService {

    @Resource
    private BasicCourseMapper basicCourseMapper;
    @Resource
    private TeachcloudCommonService teachcloudCommonService;
    @Resource
    private SysOrgService sysOrgService;
    @Resource
    private BasicUserCourseService basicUserCourseService;



    @Override
    public List<BasicCourse> listCoursesByUserId(Long userId) {
        return this.baseMapper.listCoursesByUserId(userId);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void verifyCourseInfo(Long schoolId, String courseCode, String courseName, Long userId) {
        QueryWrapper<BasicCourse> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasicCourse::getSchoolId, schoolId).eq(BasicCourse::getCode, courseCode);

        BasicCourse tmp = this.getOne(queryWrapper);
        if (tmp != null) {
            if (!courseName.equals(tmp.getName())) {
                throw ExceptionResultEnum.ERROR
                        .exception("课程代码和课程名称不匹配,课程代码为 ：'" + courseCode + "',输入的课程名称 ：'" + courseName + "',实际该代码对应的课程名称 ：'" + tmp.getName() + "'。");
            }
        } else {
            BasicCourse basicCourse = new BasicCourse();
            basicCourse.setName(courseName);
            basicCourse.setCode(courseCode);
            basicCourse.setSchoolId(schoolId);
            basicCourse.setId(SystemConstant.getDbUuid());
            basicCourse.setCreateId(userId);
            basicCourse.setCreateTime(System.currentTimeMillis());
            this.saveOrUpdate(basicCourse);
        }
    }

    @Override
    public List<CourseInfoDto> findByUserLoginNameAndRealName(String loginName, String realName) {
        System.out.println("---");
        if (loginName == null) {
            loginName = "";
        }
        if (realName == null) {
            realName = "";
        }
        return basicCourseMapper.findByUserLoginNameAndRealName(loginName, realName);
    }

    @Override
    public BasicCourse findByCourseCode(String courseCode) {
        return this.getOne(new QueryWrapper<BasicCourse>().lambda().eq(BasicCourse::getCode,courseCode));
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void createCourse(Long schoolId, String courseCode, String courseName) {
        // 向'basic_course'表新增数据 - 教研分析用
        List<BasicCourse> basicCourseList = this.list(new QueryWrapper<BasicCourse>().lambda()
                .eq(BasicCourse::getSchoolId,schoolId).eq(BasicCourse::getCode,courseCode));
        if (basicCourseList.size() > 0){
            throw ExceptionResultEnum.ERROR.exception("已存在学校id为【" + schoolId + "】,课程编号为【" + courseCode + "】的课程");
        }
        BasicCourse basicCourse = new BasicCourse();
        basicCourse.setId(SystemConstant.getDbUuid());
        basicCourse.setCode(courseCode);
        basicCourse.setName(courseName);
        basicCourse.setSchoolId(schoolId);
        this.save(basicCourse);
    }

    @Override
    public IPage<BasicCourseResult> basicCoursePage(String courseName, Long startCreateTime, Long endCreateTime, int pageNumber, int pageSize) {
        Long schoolId = SystemConstant.convertIdToLong(ServletUtil.getRequestHeaderSchoolId().toString());
        Set<Long> orgIds = teachcloudCommonService.listSubOrgIds(null);
        courseName = SystemConstant.translateSpecificSign(courseName);
        return basicCourseMapper.findBasicCoursePage(new Page<>(pageNumber,pageSize),courseName,startCreateTime,endCreateTime,schoolId,orgIds);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Long saveBasicCourse(BasicCourseParams basicCourseParams) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Long schoolId = sysUser.getSchoolId();
        Long id = basicCourseParams.getId();
        String courseName = basicCourseParams.getCourseName();
        // 校验课程编号
        String courseCode = basicCourseParams.getCourseCode();
        BasicCourse checkCode = this.getOne(new QueryWrapper<BasicCourse>().lambda()
                .eq(BasicCourse::getCode,courseCode)
                .eq(BasicCourse::getSchoolId,schoolId)
                .eq(BasicCourse::getEnable,true));
        if (Objects.nonNull(checkCode)){
            if (!checkCode.getId().equals(id)){
                throw ExceptionResultEnum.ERROR.exception("课程编号【" + courseCode +"】重复");
            }
        }
        // 校验教研室id
        Long teachingRoomId = basicCourseParams.getTeachingRoomId();
        if (Objects.isNull(sysOrgService.getOne(new QueryWrapper<SysOrg>().lambda()
                .eq(SysOrg::getId,teachingRoomId)
                .eq(SysOrg::getEnable,true)
                .eq(SysOrg::getType, OrgTypeEnum.TEACHING_ROOM)))){
            throw ExceptionResultEnum.ERROR.exception("所选教研室不存在");
        }
        // 校验班级
        List<String> clazzList = basicCourseParams.getClazzList();
        if (clazzList.size() != clazzList.stream().distinct().count()){
            throw ExceptionResultEnum.ERROR.exception("输入的班级重复");
        }

        //基础数据组装
        BasicCourse basicCourse = new BasicCourse();
        basicCourse.setName(courseName);
        basicCourse.setCode(courseCode);
        basicCourse.setTeachingRoomId(teachingRoomId);
        basicCourse.setSchoolId(schoolId);

        if (!SystemConstant.longNotNull(id)){
            // 没有id -》新增
            id = SystemConstant.getDbUuid();
            basicCourse.setId(id);
            basicCourse.setCreateId(sysUser.getId());
            basicCourse.setEnable(true);
        }else {
            // 有id -》 更新
            basicCourse.setId(id);
            basicCourse.setUpdateId(sysUser.getId());
            basicCourse.setOrgId(sysUser.getOrgId());
        }

        // 根据课程id和授课班级集合更新'basic_user_course'表
        basicUserCourseService.updateByCourseIdAndClazzList(id,clazzList,sysUser);

        // 新增或更新基础课程表
        this.saveOrUpdate(basicCourse);

        return id;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Boolean removeBasicCourseBatch(List<Long> idList) {
        if (idList.size() == 0){
            throw ExceptionResultEnum.ERROR.exception("请选择要删除的目标");
        }
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();

        UpdateWrapper<BasicCourse> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda()
                .set(BasicCourse::getEnable,false)
                .in(BasicCourse::getId,idList)
                .set(BasicCourse::getOrgId,sysUser.getOrgId())
                .set(BasicCourse::getUpdateId,sysUser.getId());

        return this.update(updateWrapper);
    }
}
